@@ -66,6 +66,7 @@ struct Example<A: hal::Api> {
     pipeline: A::RenderPipeline,
     bunnies: Vec<Locals>,
     local_buffer: A::Buffer,
+    local_alignment: wgt::BufferAddress,
     global_buffer: A::Buffer,
     sampler: A::Sampler,
     texture: A::Texture,
@@ -89,7 +90,7 @@ impl<A: hal::Api> Example<A> {
         let instance = unsafe { A::Instance::init(&instance_desc)? };
         let mut surface = unsafe { instance.create_surface(window).unwrap() };
 
-        let adapter = unsafe {
+        let (adapter, capabilities) = unsafe {
             let mut adapters = instance.enumerate_adapters();
             if adapters.is_empty() {
                 return Err(hal::InstanceError);
@@ -99,10 +100,13 @@ impl<A: hal::Api> Example<A> {
                 "Surface caps: {:?}",
                 exposed.adapter.surface_capabilities(&surface)
             );
-            exposed.adapter
+            (exposed.adapter, exposed.capabilities)
+        };
+        let hal::OpenDevice { device, mut queue } = unsafe {
+            adapter
+                .open(wgt::Features::empty(), &wgt::Limits::default())
+                .unwrap()
         };
-        let hal::OpenDevice { device, mut queue } =
-            unsafe { adapter.open(wgt::Features::empty()).unwrap() };
 
         let window_size: (u32, u32) = window.inner_size().into();
         let surface_config = hal::SurfaceConfiguration {
@@ -136,7 +140,10 @@ impl<A: hal::Api> Example<A> {
             .unwrap();
             hal::NagaShader { module, info }
         };
-        let shader_desc = hal::ShaderModuleDescriptor { label: None };
+        let shader_desc = hal::ShaderModuleDescriptor {
+            label: None,
+            runtime_checks: false,
+        };
         let shader = unsafe {
             device
                 .create_shader_module(&shader_desc, hal::ShaderInput::Naga(naga_shader))
@@ -145,6 +152,7 @@ impl<A: hal::Api> Example<A> {
 
         let global_bgl_desc = hal::BindGroupLayoutDescriptor {
             label: None,
+            flags: hal::BindGroupLayoutFlags::empty(),
             entries: &[
                 wgt::BindGroupLayoutEntry {
                     binding: 0,
@@ -182,6 +190,8 @@ impl<A: hal::Api> Example<A> {
             unsafe { device.create_bind_group_layout(&global_bgl_desc).unwrap() };
 
         let local_bgl_desc = hal::BindGroupLayoutDescriptor {
+            label: None,
+            flags: hal::BindGroupLayoutFlags::empty(),
             entries: &[wgt::BindGroupLayoutEntry {
                 binding: 0,
                 visibility: wgt::ShaderStages::VERTEX,
@@ -192,7 +202,6 @@ impl<A: hal::Api> Example<A> {
                 },
                 count: None,
             }],
-            label: None,
         };
         let local_group_layout =
             unsafe { device.create_bind_group_layout(&local_bgl_desc).unwrap() };
@@ -366,9 +375,22 @@ impl<A: hal::Api> Example<A> {
             buffer
         };
 
+        fn align_to(
+            value: wgt::BufferAddress,
+            alignment: wgt::BufferAddress,
+        ) -> wgt::BufferAddress {
+            match value % alignment {
+                0 => value,
+                other => value - other + alignment,
+            }
+        }
+        let local_alignment = align_to(
+            mem::size_of::<Locals>() as _,
+            capabilities.limits.min_uniform_buffer_offset_alignment as _,
+        );
         let local_buffer_desc = hal::BufferDescriptor {
             label: Some("local"),
-            size: (MAX_BUNNIES as wgt::BufferAddress) * wgt::BIND_BUFFER_ALIGNMENT,
+            size: (MAX_BUNNIES as wgt::BufferAddress) * local_alignment,
             usage: hal::BufferUses::MAP_WRITE | hal::BufferUses::UNIFORM,
             memory_flags: hal::MemoryFlags::PREFER_COHERENT,
         };
@@ -403,14 +425,17 @@ impl<A: hal::Api> Example<A> {
                     hal::BindGroupEntry {
                         binding: 0,
                         resource_index: 0,
+                        count: 1,
                     },
                     hal::BindGroupEntry {
                         binding: 1,
                         resource_index: 0,
+                        count: 1,
                     },
                     hal::BindGroupEntry {
                         binding: 2,
                         resource_index: 0,
+                        count: 1,
                     },
                 ],
             };
@@ -432,6 +457,7 @@ impl<A: hal::Api> Example<A> {
                 entries: &[hal::BindGroupEntry {
                     binding: 0,
                     resource_index: 0,
+                    count: 1,
                 }],
             };
             unsafe { device.create_bind_group(&local_group_desc).unwrap() }
@@ -466,6 +492,7 @@ impl<A: hal::Api> Example<A> {
             local_group_layout,
             bunnies: Vec::new(),
             local_buffer,
+            local_alignment,
             global_buffer,
             sampler,
             texture,
@@ -575,7 +602,7 @@ impl<A: hal::Api> Example<A> {
         }
 
         if !self.bunnies.is_empty() {
-            let size = self.bunnies.len() * wgt::BIND_BUFFER_ALIGNMENT as usize;
+            let size = self.bunnies.len() * self.local_alignment as usize;
             unsafe {
                 let mapping = self
                     .device
@@ -649,8 +676,7 @@ impl<A: hal::Api> Example<A> {
         }
 
         for i in 0..self.bunnies.len() {
-            let offset =
-                (i as wgt::DynamicOffset) * (wgt::BIND_BUFFER_ALIGNMENT as wgt::DynamicOffset);
+            let offset = (i as wgt::DynamicOffset) * (self.local_alignment as wgt::DynamicOffset);
             unsafe {
                 ctx.encoder
                     .set_bind_group(&self.pipeline_layout, 1, &self.local_group, &[offset]);
