@@ -7,17 +7,21 @@ use std::{ffi::CStr, mem, ptr, sync::Arc};
 
 //TODO: const fn?
 fn indexing_features() -> wgt::Features {
-    wgt::Features::BUFFER_BINDING_ARRAY | wgt::Features::TEXTURE_BINDING_ARRAY
+    wgt::Features::SAMPLED_TEXTURE_AND_STORAGE_BUFFER_ARRAY_NON_UNIFORM_INDEXING
+        | wgt::Features::UNIFORM_BUFFER_AND_STORAGE_TEXTURE_ARRAY_NON_UNIFORM_INDEXING
+        | wgt::Features::UNSIZED_BINDING_ARRAY
 }
 
 /// Aggregate of the `vk::PhysicalDevice*Features` structs used by `gfx`.
 #[derive(Debug, Default)]
 pub struct PhysicalDeviceFeatures {
     core: vk::PhysicalDeviceFeatures,
-    vulkan_1_2: Option<vk::PhysicalDeviceVulkan12Features>,
-    descriptor_indexing: Option<vk::PhysicalDeviceDescriptorIndexingFeaturesEXT>,
+    pub(super) vulkan_1_2: Option<vk::PhysicalDeviceVulkan12Features>,
+    pub(super) descriptor_indexing: Option<vk::PhysicalDeviceDescriptorIndexingFeaturesEXT>,
     imageless_framebuffer: Option<vk::PhysicalDeviceImagelessFramebufferFeaturesKHR>,
     timeline_semaphore: Option<vk::PhysicalDeviceTimelineSemaphoreFeaturesKHR>,
+    image_robustness: Option<vk::PhysicalDeviceImageRobustnessFeaturesEXT>,
+    robustness2: Option<vk::PhysicalDeviceRobustness2FeaturesEXT>,
 }
 
 // This is safe because the structs have `p_next: *mut c_void`, which we null out/never read.
@@ -52,14 +56,34 @@ impl PhysicalDeviceFeatures {
         requested_features: wgt::Features,
         downlevel_flags: wgt::DownlevelFlags,
         private_caps: &super::PrivateCapabilities,
+        uab_types: super::UpdateAfterBindTypes,
     ) -> Self {
-        //TODO: make configurable
-        let rba = !(cfg!(target_os = "macos") || cfg!(target_os = "ios"));
+        let needs_sampled_image_non_uniform = requested_features.contains(
+            wgt::Features::TEXTURE_BINDING_ARRAY
+                | wgt::Features::SAMPLED_TEXTURE_AND_STORAGE_BUFFER_ARRAY_NON_UNIFORM_INDEXING,
+        );
+        let needs_storage_buffer_non_uniform = requested_features.contains(
+            wgt::Features::BUFFER_BINDING_ARRAY
+                | wgt::Features::STORAGE_RESOURCE_BINDING_ARRAY
+                | wgt::Features::SAMPLED_TEXTURE_AND_STORAGE_BUFFER_ARRAY_NON_UNIFORM_INDEXING,
+        );
+        let needs_uniform_buffer_non_uniform = requested_features.contains(
+            wgt::Features::TEXTURE_BINDING_ARRAY
+                | wgt::Features::UNIFORM_BUFFER_AND_STORAGE_TEXTURE_ARRAY_NON_UNIFORM_INDEXING,
+        );
+        let needs_storage_image_non_uniform = requested_features.contains(
+            wgt::Features::TEXTURE_BINDING_ARRAY
+                | wgt::Features::STORAGE_RESOURCE_BINDING_ARRAY
+                | wgt::Features::UNIFORM_BUFFER_AND_STORAGE_TEXTURE_ARRAY_NON_UNIFORM_INDEXING,
+        );
+        let needs_partially_bound =
+            requested_features.intersects(wgt::Features::PARTIALLY_BOUND_BINDING_ARRAY);
+
         Self {
             // vk::PhysicalDeviceFeatures is a struct composed of Bool32's while
             // Features is a bitfield so we need to map everything manually
             core: vk::PhysicalDeviceFeatures::builder()
-                .robust_buffer_access(rba)
+                .robust_buffer_access(private_caps.robust_buffer_access)
                 .independent_blend(true)
                 .sample_rate_shading(true)
                 .image_cube_array(
@@ -70,9 +94,9 @@ impl PhysicalDeviceFeatures {
                     requested_features.contains(wgt::Features::MULTI_DRAW_INDIRECT),
                 )
                 .depth_clamp(requested_features.contains(wgt::Features::DEPTH_CLAMPING))
-                .fill_mode_non_solid(
-                    requested_features.contains(wgt::Features::NON_FILL_POLYGON_MODE),
-                )
+                .fill_mode_non_solid(requested_features.intersects(
+                    wgt::Features::POLYGON_MODE_LINE | wgt::Features::POLYGON_MODE_POINT,
+                ))
                 //.depth_bounds(requested_features.contains(wgt::Features::DEPTH_BOUNDS))
                 //.alpha_to_one(requested_features.contains(wgt::Features::ALPHA_TO_ONE))
                 //.multi_viewport(requested_features.contains(wgt::Features::MULTI_VIEWPORTS))
@@ -132,32 +156,30 @@ impl PhysicalDeviceFeatures {
                         )
                         .descriptor_indexing(requested_features.intersects(indexing_features()))
                         .shader_sampled_image_array_non_uniform_indexing(
-                            requested_features.contains(
-                                wgt::Features::TEXTURE_BINDING_ARRAY
-                                    | wgt::Features::SAMPLED_TEXTURE_AND_STORAGE_BUFFER_ARRAY_NON_UNIFORM_INDEXING,
-                            ),
+                            needs_sampled_image_non_uniform,
                         )
                         .shader_storage_image_array_non_uniform_indexing(
-                            requested_features.contains(
-                                wgt::Features::TEXTURE_BINDING_ARRAY
-                                    | wgt::Features::STORAGE_RESOURCE_BINDING_ARRAY
-                                    | wgt::Features::UNIFORM_BUFFER_AND_STORAGE_TEXTURE_ARRAY_NON_UNIFORM_INDEXING,
-                            ),
+                            needs_storage_image_non_uniform,
                         )
-                        //.shader_storage_buffer_array_non_uniform_indexing(
                         .shader_uniform_buffer_array_non_uniform_indexing(
-                            requested_features.contains(
-                                wgt::Features::BUFFER_BINDING_ARRAY
-                                    | wgt::Features::UNIFORM_BUFFER_AND_STORAGE_TEXTURE_ARRAY_NON_UNIFORM_INDEXING,
-                            ),
+                            needs_uniform_buffer_non_uniform,
                         )
                         .shader_storage_buffer_array_non_uniform_indexing(
-                            requested_features.contains(
-                                wgt::Features::BUFFER_BINDING_ARRAY
-                                    | wgt::Features::STORAGE_RESOURCE_BINDING_ARRAY
-                                    | wgt::Features::SAMPLED_TEXTURE_AND_STORAGE_BUFFER_ARRAY_NON_UNIFORM_INDEXING,
-                            ),
+                            needs_storage_buffer_non_uniform,
                         )
+                        .descriptor_binding_sampled_image_update_after_bind(
+                            uab_types.contains(super::UpdateAfterBindTypes::SAMPLED_TEXTURE),
+                        )
+                        .descriptor_binding_storage_image_update_after_bind(
+                            uab_types.contains(super::UpdateAfterBindTypes::STORAGE_TEXTURE),
+                        )
+                        .descriptor_binding_uniform_buffer_update_after_bind(
+                            uab_types.contains(super::UpdateAfterBindTypes::UNIFORM_BUFFER),
+                        )
+                        .descriptor_binding_storage_buffer_update_after_bind(
+                            uab_types.contains(super::UpdateAfterBindTypes::STORAGE_BUFFER),
+                        )
+                        .descriptor_binding_partially_bound(needs_partially_bound)
                         .runtime_descriptor_array(
                             requested_features.contains(wgt::Features::UNSIZED_BINDING_ARRAY),
                         )
@@ -175,32 +197,30 @@ impl PhysicalDeviceFeatures {
                 Some(
                     vk::PhysicalDeviceDescriptorIndexingFeaturesEXT::builder()
                         .shader_sampled_image_array_non_uniform_indexing(
-                            requested_features.contains(
-                                wgt::Features::TEXTURE_BINDING_ARRAY
-                                    | wgt::Features::SAMPLED_TEXTURE_AND_STORAGE_BUFFER_ARRAY_NON_UNIFORM_INDEXING,
-                            ),
+                            needs_sampled_image_non_uniform,
                         )
                         .shader_storage_image_array_non_uniform_indexing(
-                            requested_features.contains(
-                                wgt::Features::TEXTURE_BINDING_ARRAY
-                                    | wgt::Features::STORAGE_RESOURCE_BINDING_ARRAY
-                                    | wgt::Features::UNIFORM_BUFFER_AND_STORAGE_TEXTURE_ARRAY_NON_UNIFORM_INDEXING,
-                            ),
+                            needs_storage_image_non_uniform,
                         )
-                        //.shader_storage_buffer_array_non_uniform_indexing(
                         .shader_uniform_buffer_array_non_uniform_indexing(
-                            requested_features.contains(
-                                wgt::Features::BUFFER_BINDING_ARRAY
-                                    | wgt::Features::UNIFORM_BUFFER_AND_STORAGE_TEXTURE_ARRAY_NON_UNIFORM_INDEXING,
-                            ),
+                            needs_uniform_buffer_non_uniform,
                         )
                         .shader_storage_buffer_array_non_uniform_indexing(
-                            requested_features.contains(
-                                wgt::Features::BUFFER_BINDING_ARRAY
-                                    | wgt::Features::STORAGE_RESOURCE_BINDING_ARRAY
-                                    | wgt::Features::SAMPLED_TEXTURE_AND_STORAGE_BUFFER_ARRAY_NON_UNIFORM_INDEXING,
-                            ),
+                            needs_storage_buffer_non_uniform,
                         )
+                        .descriptor_binding_sampled_image_update_after_bind(
+                            uab_types.contains(super::UpdateAfterBindTypes::SAMPLED_TEXTURE),
+                        )
+                        .descriptor_binding_storage_image_update_after_bind(
+                            uab_types.contains(super::UpdateAfterBindTypes::STORAGE_TEXTURE),
+                        )
+                        .descriptor_binding_uniform_buffer_update_after_bind(
+                            uab_types.contains(super::UpdateAfterBindTypes::UNIFORM_BUFFER),
+                        )
+                        .descriptor_binding_storage_buffer_update_after_bind(
+                            uab_types.contains(super::UpdateAfterBindTypes::STORAGE_BUFFER),
+                        )
+                        .descriptor_binding_partially_bound(needs_partially_bound)
                         .runtime_descriptor_array(
                             requested_features.contains(wgt::Features::UNSIZED_BINDING_ARRAY),
                         )
@@ -230,10 +250,33 @@ impl PhysicalDeviceFeatures {
             } else {
                 None
             },
+            image_robustness: if enabled_extensions.contains(&vk::ExtImageRobustnessFn::name()) {
+                Some(
+                    vk::PhysicalDeviceImageRobustnessFeaturesEXT::builder()
+                        .robust_image_access(private_caps.robust_image_access)
+                        .build(),
+                )
+            } else {
+                None
+            },
+            robustness2: if enabled_extensions.contains(&vk::ExtRobustness2Fn::name()) {
+                // Note: enabling `robust_buffer_access2` isn't requires, strictly speaking
+                // since we can enable `robust_buffer_access` all the time. But it improves
+                // program portability, so we opt into it anyway.
+                Some(
+                    vk::PhysicalDeviceRobustness2FeaturesEXT::builder()
+                        .robust_buffer_access2(private_caps.robust_buffer_access)
+                        .robust_image_access2(private_caps.robust_image_access)
+                        .build(),
+                )
+            } else {
+                None
+            },
         }
     }
 
     fn to_wgpu(&self, caps: &PhysicalDeviceCapabilities) -> (wgt::Features, wgt::DownlevelFlags) {
+        use crate::auxil::db;
         use wgt::{DownlevelFlags as Df, Features as F};
         let mut features = F::empty()
             | F::SPIRV_SHADER_PASSTHROUGH
@@ -242,7 +285,8 @@ impl PhysicalDeviceFeatures {
             | F::ADDRESS_MODE_CLAMP_TO_BORDER
             | F::TIMESTAMP_QUERY
             | F::PIPELINE_STATISTICS_QUERY
-            | F::TEXTURE_ADAPTER_SPECIFIC_FORMAT_FEATURES;
+            | F::TEXTURE_ADAPTER_SPECIFIC_FORMAT_FEATURES
+            | F::CLEAR_COMMANDS;
         let mut dl_flags = Df::all();
 
         dl_flags.set(Df::CUBE_ARRAY_TEXTURES, self.core.image_cube_array != 0);
@@ -255,7 +299,8 @@ impl PhysicalDeviceFeatures {
         //if self.core.dual_src_blend != 0
         features.set(F::MULTI_DRAW_INDIRECT, self.core.multi_draw_indirect != 0);
         features.set(F::DEPTH_CLAMPING, self.core.depth_clamp != 0);
-        features.set(F::NON_FILL_POLYGON_MODE, self.core.fill_mode_non_solid != 0);
+        features.set(F::POLYGON_MODE_LINE, self.core.fill_mode_non_solid != 0);
+        features.set(F::POLYGON_MODE_POINT, self.core.fill_mode_non_solid != 0);
         //if self.core.depth_bounds != 0 {
         //if self.core.alpha_to_one != 0 {
         //if self.core.multi_viewport != 0 {
@@ -321,6 +366,8 @@ impl PhysicalDeviceFeatures {
             caps.supports_extension(vk::ExtConservativeRasterizationFn::name()),
         );
 
+        let intel_windows = caps.properties.vendor_id == db::intel::VENDOR && cfg!(windows);
+
         if let Some(ref vulkan_1_2) = self.vulkan_1_2 {
             const STORAGE: F = F::STORAGE_RESOURCE_BINDING_ARRAY;
             if Self::all_features_supported(
@@ -356,6 +403,9 @@ impl PhysicalDeviceFeatures {
             if vulkan_1_2.runtime_descriptor_array != 0 {
                 features |= F::UNSIZED_BINDING_ARRAY;
             }
+            if vulkan_1_2.descriptor_binding_partially_bound != 0 && !intel_windows {
+                features |= F::PARTIALLY_BOUND_BINDING_ARRAY;
+            }
             //if vulkan_1_2.sampler_mirror_clamp_to_edge != 0 {
             //if vulkan_1_2.sampler_filter_minmax != 0 {
             if vulkan_1_2.draw_indirect_count != 0 {
@@ -395,6 +445,9 @@ impl PhysicalDeviceFeatures {
             ) {
                 features.insert(F::UNIFORM_BUFFER_AND_STORAGE_TEXTURE_ARRAY_NON_UNIFORM_INDEXING);
             }
+            if descriptor_indexing.descriptor_binding_partially_bound != 0 && !intel_windows {
+                features |= F::PARTIALLY_BOUND_BINDING_ARRAY;
+            }
             if descriptor_indexing.runtime_descriptor_array != 0 {
                 features |= F::UNSIZED_BINDING_ARRAY;
             }
@@ -414,11 +467,18 @@ impl PhysicalDeviceFeatures {
 }
 
 /// Information gathered about a physical device capabilities.
+#[derive(Default)]
 pub struct PhysicalDeviceCapabilities {
     supported_extensions: Vec<vk::ExtensionProperties>,
     properties: vk::PhysicalDeviceProperties,
+    vulkan_1_2: Option<vk::PhysicalDeviceVulkan12Properties>,
+    descriptor_indexing: Option<vk::PhysicalDeviceDescriptorIndexingPropertiesEXT>,
 }
 
+// This is safe because the structs have `p_next: *mut c_void`, which we null out/never read.
+unsafe impl Send for PhysicalDeviceCapabilities {}
+unsafe impl Sync for PhysicalDeviceCapabilities {}
+
 impl PhysicalDeviceCapabilities {
     fn supports_extension(&self, extension: &CStr) -> bool {
         self.supported_extensions
@@ -474,8 +534,63 @@ impl PhysicalDeviceCapabilities {
         extensions
     }
 
-    fn to_wgpu_limits(&self) -> wgt::Limits {
+    fn to_wgpu_limits(&self, features: &PhysicalDeviceFeatures) -> wgt::Limits {
         let limits = &self.properties.limits;
+
+        let uab_types = super::UpdateAfterBindTypes::from_features(features);
+
+        let max_sampled_textures =
+            if uab_types.contains(super::UpdateAfterBindTypes::SAMPLED_TEXTURE) {
+                if let Some(di) = self.descriptor_indexing {
+                    di.max_per_stage_descriptor_update_after_bind_sampled_images
+                } else if let Some(vk_1_2) = self.vulkan_1_2 {
+                    vk_1_2.max_per_stage_descriptor_update_after_bind_sampled_images
+                } else {
+                    limits.max_per_stage_descriptor_sampled_images
+                }
+            } else {
+                limits.max_per_stage_descriptor_sampled_images
+            };
+
+        let max_storage_textures =
+            if uab_types.contains(super::UpdateAfterBindTypes::STORAGE_TEXTURE) {
+                if let Some(di) = self.descriptor_indexing {
+                    di.max_per_stage_descriptor_update_after_bind_storage_images
+                } else if let Some(vk_1_2) = self.vulkan_1_2 {
+                    vk_1_2.max_per_stage_descriptor_update_after_bind_storage_images
+                } else {
+                    limits.max_per_stage_descriptor_storage_images
+                }
+            } else {
+                limits.max_per_stage_descriptor_storage_images
+            };
+
+        let max_uniform_buffers = if uab_types.contains(super::UpdateAfterBindTypes::UNIFORM_BUFFER)
+        {
+            if let Some(di) = self.descriptor_indexing {
+                di.max_per_stage_descriptor_update_after_bind_uniform_buffers
+            } else if let Some(vk_1_2) = self.vulkan_1_2 {
+                vk_1_2.max_per_stage_descriptor_update_after_bind_uniform_buffers
+            } else {
+                limits.max_per_stage_descriptor_uniform_buffers
+            }
+        } else {
+            limits.max_per_stage_descriptor_uniform_buffers
+        };
+
+        let max_storage_buffers = if uab_types.contains(super::UpdateAfterBindTypes::STORAGE_BUFFER)
+        {
+            if let Some(di) = self.descriptor_indexing {
+                di.max_per_stage_descriptor_update_after_bind_storage_buffers
+            } else if let Some(vk_1_2) = self.vulkan_1_2 {
+                vk_1_2.max_per_stage_descriptor_update_after_bind_storage_buffers
+            } else {
+                limits.max_per_stage_descriptor_storage_buffers
+            }
+        } else {
+            limits.max_per_stage_descriptor_storage_buffers
+        };
+
         wgt::Limits {
             max_texture_dimension_1d: limits.max_image_dimension1_d,
             max_texture_dimension_2d: limits.max_image_dimension2_d,
@@ -488,11 +603,11 @@ impl PhysicalDeviceCapabilities {
                 .max_descriptor_set_uniform_buffers_dynamic,
             max_dynamic_storage_buffers_per_pipeline_layout: limits
                 .max_descriptor_set_storage_buffers_dynamic,
-            max_sampled_textures_per_shader_stage: limits.max_per_stage_descriptor_sampled_images,
+            max_sampled_textures_per_shader_stage: max_sampled_textures,
             max_samplers_per_shader_stage: limits.max_per_stage_descriptor_samplers,
-            max_storage_buffers_per_shader_stage: limits.max_per_stage_descriptor_storage_buffers,
-            max_storage_textures_per_shader_stage: limits.max_per_stage_descriptor_storage_images,
-            max_uniform_buffers_per_shader_stage: limits.max_per_stage_descriptor_uniform_buffers,
+            max_storage_buffers_per_shader_stage: max_storage_buffers,
+            max_storage_textures_per_shader_stage: max_storage_textures,
+            max_uniform_buffers_per_shader_stage: max_uniform_buffers,
             max_uniform_buffer_binding_size: limits.max_uniform_buffer_range,
             max_storage_buffer_binding_size: limits.max_storage_buffer_range,
             max_vertex_buffers: limits
@@ -501,6 +616,8 @@ impl PhysicalDeviceCapabilities {
             max_vertex_attributes: limits.max_vertex_input_attributes,
             max_vertex_buffer_array_stride: limits.max_vertex_input_binding_stride,
             max_push_constant_size: limits.max_push_constants_size,
+            min_uniform_buffer_offset_alignment: limits.min_uniform_buffer_offset_alignment as u32,
+            min_storage_buffer_offset_alignment: limits.min_storage_buffer_offset_alignment as u32,
         }
     }
 
@@ -511,10 +628,6 @@ impl PhysicalDeviceCapabilities {
                 .unwrap(),
             buffer_copy_pitch: wgt::BufferSize::new(limits.optimal_buffer_copy_row_pitch_alignment)
                 .unwrap(),
-            storage_buffer_offset: wgt::BufferSize::new(limits.min_storage_buffer_offset_alignment)
-                .unwrap(),
-            uniform_buffer_offset: wgt::BufferSize::new(limits.min_uniform_buffer_offset_alignment)
-                .unwrap(),
         }
     }
 }
@@ -525,11 +638,46 @@ impl super::InstanceShared {
         &self,
         phd: vk::PhysicalDevice,
     ) -> (PhysicalDeviceCapabilities, PhysicalDeviceFeatures) {
-        let capabilities = unsafe {
-            PhysicalDeviceCapabilities {
-                supported_extensions: self.raw.enumerate_device_extension_properties(phd).unwrap(),
-                properties: self.raw.get_physical_device_properties(phd),
-            }
+        let capabilities = {
+            let mut capabilities = PhysicalDeviceCapabilities::default();
+            capabilities.supported_extensions =
+                unsafe { self.raw.enumerate_device_extension_properties(phd).unwrap() };
+            capabilities.properties = if let Some(ref get_device_properties) =
+                self.get_physical_device_properties
+            {
+                let core = vk::PhysicalDeviceProperties::builder().build();
+                let mut properites2 = vk::PhysicalDeviceProperties2::builder()
+                    .properties(core)
+                    .build();
+
+                if capabilities.properties.api_version >= vk::API_VERSION_1_2 {
+                    capabilities.vulkan_1_2 =
+                        Some(vk::PhysicalDeviceVulkan12Properties::builder().build());
+
+                    let mut_ref = capabilities.vulkan_1_2.as_mut().unwrap();
+                    mut_ref.p_next =
+                        mem::replace(&mut properites2.p_next, mut_ref as *mut _ as *mut _);
+                }
+
+                if capabilities.supports_extension(vk::ExtDescriptorIndexingFn::name()) {
+                    capabilities.descriptor_indexing =
+                        Some(vk::PhysicalDeviceDescriptorIndexingPropertiesEXT::builder().build());
+
+                    let mut_ref = capabilities.descriptor_indexing.as_mut().unwrap();
+                    mut_ref.p_next =
+                        mem::replace(&mut properites2.p_next, mut_ref as *mut _ as *mut _);
+                }
+
+                unsafe {
+                    get_device_properties
+                        .get_physical_device_properties2_khr(phd, &mut properites2);
+                }
+                properites2.properties
+            } else {
+                unsafe { self.raw.get_physical_device_properties(phd) }
+            };
+
+            capabilities
         };
 
         let mut features = PhysicalDeviceFeatures::default();
@@ -573,6 +721,21 @@ impl super::InstanceShared {
                 mut_ref.p_next = mem::replace(&mut features2.p_next, mut_ref as *mut _ as *mut _);
             }
 
+            if capabilities.supports_extension(vk::ExtImageRobustnessFn::name()) {
+                features.image_robustness =
+                    Some(vk::PhysicalDeviceImageRobustnessFeaturesEXT::builder().build());
+
+                let mut_ref = features.image_robustness.as_mut().unwrap();
+                mut_ref.p_next = mem::replace(&mut features2.p_next, mut_ref as *mut _ as *mut _);
+            }
+            if capabilities.supports_extension(vk::ExtRobustness2Fn::name()) {
+                features.robustness2 =
+                    Some(vk::PhysicalDeviceRobustness2FeaturesEXT::builder().build());
+
+                let mut_ref = features.robustness2.as_mut().unwrap();
+                mut_ref.p_next = mem::replace(&mut features2.p_next, mut_ref as *mut _ as *mut _);
+            }
+
             unsafe {
                 get_device_properties.get_physical_device_features2_khr(phd, &mut features2);
             }
@@ -595,6 +758,8 @@ impl super::InstanceShared {
             null_p_next(&mut features.descriptor_indexing);
             null_p_next(&mut features.imageless_framebuffer);
             null_p_next(&mut features.timeline_semaphore);
+            null_p_next(&mut features.image_robustness);
+            null_p_next(&mut features.robustness2);
         }
 
         (capabilities, features)
@@ -606,6 +771,8 @@ impl super::Instance {
         &self,
         phd: vk::PhysicalDevice,
     ) -> Option<crate::ExposedAdapter<super::Api>> {
+        use crate::auxil::db;
+
         let (phd_capabilities, phd_features) = self.shared.inspect(phd);
 
         let info = wgt::AdapterInfo {
@@ -629,8 +796,8 @@ impl super::Instance {
         };
 
         let (available_features, downlevel_flags) = phd_features.to_wgpu(&phd_capabilities);
+        let mut workarounds = super::Workarounds::empty();
         {
-            use crate::auxil::db;
             // see https://github.com/gfx-rs/gfx/issues/1930
             let _is_windows_intel_dual_src_bug = cfg!(windows)
                 && phd_capabilities.properties.vendor_id == db::intel::VENDOR
@@ -638,6 +805,8 @@ impl super::Instance {
                     == db::intel::DEVICE_KABY_LAKE_MASK
                     || phd_capabilities.properties.device_id & db::intel::DEVICE_SKY_LAKE_MASK
                         == db::intel::DEVICE_SKY_LAKE_MASK);
+            // TODO: only enable for particular devices
+            workarounds |= super::Workarounds::SEPARATE_ENTRY_POINTS;
         };
 
         if phd_features.core.sample_rate_shading == 0 {
@@ -674,13 +843,13 @@ impl super::Instance {
                 || phd_capabilities.supports_extension(vk::KhrMaintenance1Fn::name()),
             imageless_framebuffers: match phd_features.vulkan_1_2 {
                 Some(features) => features.imageless_framebuffer == vk::TRUE,
-                None => phd_capabilities.supports_extension(vk::KhrImagelessFramebufferFn::name()),
+                None => phd_features.imageless_framebuffer.is_some(),
             },
             image_view_usage: phd_capabilities.properties.api_version >= vk::API_VERSION_1_1
                 || phd_capabilities.supports_extension(vk::KhrMaintenance2Fn::name()),
             timeline_semaphores: match phd_features.vulkan_1_2 {
                 Some(features) => features.timeline_semaphore == vk::TRUE,
-                None => phd_capabilities.supports_extension(vk::KhrTimelineSemaphoreFn::name()),
+                None => phd_features.timeline_semaphore.is_some(),
             },
             texture_d24: unsafe {
                 self.shared
@@ -698,10 +867,19 @@ impl super::Instance {
             },
             non_coherent_map_mask: phd_capabilities.properties.limits.non_coherent_atom_size - 1,
             can_present: true,
+            //TODO: make configurable
+            robust_buffer_access: phd_features.core.robust_buffer_access != 0,
+            robust_image_access: match phd_features.robustness2 {
+                Some(ref f) => f.robust_image_access2 != 0,
+                None => match phd_features.image_robustness {
+                    Some(ref f) => f.robust_image_access != 0,
+                    None => false,
+                },
+            },
         };
 
         let capabilities = crate::Capabilities {
-            limits: phd_capabilities.to_wgpu_limits(),
+            limits: phd_capabilities.to_wgpu_limits(&phd_features),
             alignments: phd_capabilities.to_hal_alignments(),
             downlevel: wgt::DownlevelCapabilities {
                 flags: downlevel_flags,
@@ -723,6 +901,7 @@ impl super::Instance {
             //phd_features,
             downlevel_flags,
             private_caps,
+            workarounds,
         };
 
         Some(crate::ExposedAdapter {
@@ -757,6 +936,7 @@ impl super::Adapter {
         &self,
         enabled_extensions: &[&'static CStr],
         features: wgt::Features,
+        uab_types: super::UpdateAfterBindTypes,
     ) -> PhysicalDeviceFeatures {
         PhysicalDeviceFeatures::from_extensions_and_requested_features(
             self.phd_capabilities.properties.api_version,
@@ -764,6 +944,7 @@ impl super::Adapter {
             features,
             self.downlevel_flags,
             &self.private_caps,
+            uab_types,
         )
     }
 
@@ -772,17 +953,22 @@ impl super::Adapter {
     /// - `raw_device` must be created from this adapter.
     /// - `raw_device` must be created using `family_index`, `enabled_extensions` and `physical_device_features()`
     /// - `enabled_extensions` must be a superset of `required_device_extensions()`.
+    #[allow(clippy::too_many_arguments)]
     pub unsafe fn device_from_raw(
         &self,
         raw_device: ash::Device,
+        handle_is_owned: bool,
         enabled_extensions: &[&'static CStr],
+        uab_types: super::UpdateAfterBindTypes,
         family_index: u32,
         queue_index: u32,
     ) -> Result<crate::OpenDevice<super::Api>, crate::DeviceError> {
-        let mem_properties = self
-            .instance
-            .raw
-            .get_physical_device_memory_properties(self.raw);
+        let mem_properties = {
+            profiling::scope!("vkGetPhysicalDeviceMemoryProperties");
+            self.instance
+                .raw
+                .get_physical_device_memory_properties(self.raw)
+        };
         let memory_types =
             &mem_properties.memory_types[..mem_properties.memory_type_count as usize];
         let valid_ash_memory_types = memory_types.iter().enumerate().fold(0, |u, (i, mem)| {
@@ -842,28 +1028,53 @@ impl super::Adapter {
                 spv::WriterFlags::LABEL_VARYINGS,
                 self.phd_capabilities.properties.vendor_id != crate::auxil::db::qualcomm::VENDOR,
             );
+            flags.set(
+                spv::WriterFlags::FORCE_POINT_SIZE,
+                //Note: we could technically disable this when we are compiling separate entry points,
+                // and we know exactly that the primitive topology is not `PointList`.
+                // But this requires cloning the `spv::Options` struct, which has heap allocations.
+                true, // could check `super::Workarounds::SEPARATE_ENTRY_POINTS`
+            );
             spv::Options {
                 lang_version: (1, 0),
                 flags,
                 capabilities: Some(capabilities.iter().cloned().collect()),
-                index_bounds_check_policy: naga::back::IndexBoundsCheckPolicy::Restrict,
+                bounds_check_policies: naga::back::BoundsCheckPolicies {
+                    index: naga::back::BoundsCheckPolicy::Restrict,
+                    buffer: if self.private_caps.robust_buffer_access {
+                        naga::back::BoundsCheckPolicy::Unchecked
+                    } else {
+                        naga::back::BoundsCheckPolicy::Restrict
+                    },
+                    image: if self.private_caps.robust_image_access {
+                        naga::back::BoundsCheckPolicy::Unchecked
+                    } else {
+                        naga::back::BoundsCheckPolicy::Restrict
+                    },
+                },
             }
         };
 
         log::info!("Private capabilities: {:?}", self.private_caps);
-        let raw_queue = raw_device.get_device_queue(family_index, queue_index);
+        let raw_queue = {
+            profiling::scope!("vkGetDeviceQueue");
+            raw_device.get_device_queue(family_index, queue_index)
+        };
 
         let shared = Arc::new(super::DeviceShared {
             raw: raw_device,
+            handle_is_owned,
             instance: Arc::clone(&self.instance),
             extension_fns: super::DeviceExtensionFunctions {
                 draw_indirect_count: indirect_count_fn,
                 timeline_semaphore: timeline_semaphore_fn,
             },
             vendor_id: self.phd_capabilities.properties.vendor_id,
+            timestamp_period: self.phd_capabilities.properties.limits.timestamp_period,
+            uab_types,
             downlevel_flags: self.downlevel_flags,
             private_caps: self.private_caps.clone(),
-            timestamp_period: self.phd_capabilities.properties.limits.timestamp_period,
+            workarounds: self.workarounds,
             render_passes: Mutex::new(Default::default()),
             framebuffers: Mutex::new(Default::default()),
         });
@@ -905,7 +1116,15 @@ impl super::Adapter {
             };
             gpu_alloc::GpuAllocator::new(config, properties)
         };
-        let desc_allocator = gpu_descriptor::DescriptorAllocator::new(0);
+        let desc_allocator = gpu_descriptor::DescriptorAllocator::new(
+            if let Some(vk_12) = self.phd_capabilities.vulkan_1_2 {
+                vk_12.max_update_after_bind_descriptors_in_all_pools
+            } else if let Some(di) = self.phd_capabilities.descriptor_indexing {
+                di.max_update_after_bind_descriptors_in_all_pools
+            } else {
+                0
+            },
+        );
 
         let device = super::Device {
             shared,
@@ -925,9 +1144,14 @@ impl crate::Adapter<super::Api> for super::Adapter {
     unsafe fn open(
         &self,
         features: wgt::Features,
+        limits: &wgt::Limits,
     ) -> Result<crate::OpenDevice<super::Api>, crate::DeviceError> {
+        let phd_limits = &self.phd_capabilities.properties.limits;
+        let uab_types = super::UpdateAfterBindTypes::from_limits(limits, phd_limits);
+
         let enabled_extensions = self.required_device_extensions(features);
-        let mut enabled_phd_features = self.physical_device_features(&enabled_extensions, features);
+        let mut enabled_phd_features =
+            self.physical_device_features(&enabled_extensions, features, uab_types);
 
         let family_index = 0; //TODO
         let family_info = vk::DeviceQueueCreateInfo::builder()
@@ -950,11 +1174,16 @@ impl crate::Adapter<super::Api> for super::Adapter {
         let info = enabled_phd_features
             .add_to_device_create_builder(pre_info)
             .build();
-        let raw_device = self.instance.raw.create_device(self.raw, &info, None)?;
+        let raw_device = {
+            profiling::scope!("vkCreateDevice");
+            self.instance.raw.create_device(self.raw, &info, None)?
+        };
 
         self.device_from_raw(
             raw_device,
+            true,
             &enabled_extensions,
+            uab_types,
             family_info.queue_family_index,
             0,
         )
@@ -1029,27 +1258,33 @@ impl crate::Adapter<super::Api> for super::Adapter {
         }
 
         let queue_family_index = 0; //TODO
-        match surface.functor.get_physical_device_surface_support(
-            self.raw,
-            queue_family_index,
-            surface.raw,
-        ) {
-            Ok(true) => (),
-            Ok(false) => return None,
-            Err(e) => {
-                log::error!("get_physical_device_surface_support: {}", e);
-                return None;
+        {
+            profiling::scope!("vkGetPhysicalDeviceSurfaceSupportKHR");
+            match surface.functor.get_physical_device_surface_support(
+                self.raw,
+                queue_family_index,
+                surface.raw,
+            ) {
+                Ok(true) => (),
+                Ok(false) => return None,
+                Err(e) => {
+                    log::error!("get_physical_device_surface_support: {}", e);
+                    return None;
+                }
             }
         }
 
-        let caps = match surface
-            .functor
-            .get_physical_device_surface_capabilities(self.raw, surface.raw)
-        {
-            Ok(caps) => caps,
-            Err(e) => {
-                log::error!("get_physical_device_surface_capabilities: {}", e);
-                return None;
+        let caps = {
+            profiling::scope!("vkGetPhysicalDeviceSurfaceCapabilitiesKHR");
+            match surface
+                .functor
+                .get_physical_device_surface_capabilities(self.raw, surface.raw)
+            {
+                Ok(caps) => caps,
+                Err(e) => {
+                    log::error!("get_physical_device_surface_capabilities: {}", e);
+                    return None;
+                }
             }
         };
 
@@ -1084,25 +1319,31 @@ impl crate::Adapter<super::Api> for super::Adapter {
             depth_or_array_layers: caps.max_image_array_layers,
         };
 
-        let raw_present_modes = match surface
-            .functor
-            .get_physical_device_surface_present_modes(self.raw, surface.raw)
-        {
-            Ok(present_modes) => present_modes,
-            Err(e) => {
-                log::error!("get_physical_device_surface_present_modes: {}", e);
-                Vec::new()
+        let raw_present_modes = {
+            profiling::scope!("vkGetPhysicalDeviceSurfacePresentModesKHR");
+            match surface
+                .functor
+                .get_physical_device_surface_present_modes(self.raw, surface.raw)
+            {
+                Ok(present_modes) => present_modes,
+                Err(e) => {
+                    log::error!("get_physical_device_surface_present_modes: {}", e);
+                    Vec::new()
+                }
             }
         };
 
-        let raw_surface_formats = match surface
-            .functor
-            .get_physical_device_surface_formats(self.raw, surface.raw)
-        {
-            Ok(formats) => formats,
-            Err(e) => {
-                log::error!("get_physical_device_surface_formats: {}", e);
-                Vec::new()
+        let raw_surface_formats = {
+            profiling::scope!("vkGetPhysicalDeviceSurfaceFormatsKHR");
+            match surface
+                .functor
+                .get_physical_device_surface_formats(self.raw, surface.raw)
+            {
+                Ok(formats) => formats,
+                Err(e) => {
+                    log::error!("get_physical_device_surface_formats: {}", e);
+                    Vec::new()
+                }
             }
         };
 
